#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include "linear.h"
#include "tron.h"
#include <time.h>
typedef signed char schar;
template <class T> inline void swap(T& x, T& y) { T t=x; x=y; y=t; }
#ifndef min
template <class T> inline T min(T x,T y) { return (x<y)?x:y; }
#endif
#ifndef max
template <class T> inline T max(T x,T y) { return (x>y)?x:y; }
#endif

#define Malloc(type,n) (type *)malloc((n)*sizeof(type))
#define INF HUGE_VAL

#ifdef __cplusplus
extern "C" {
#endif
	extern double dnrm2_(int *, double *, int *);
#ifdef __cplusplus
}
#endif


#if 1
void info(const char *fmt,...)
{
	va_list ap;
	va_start(ap,fmt);
	vprintf(fmt,ap);
	va_end(ap);
}
void info_flush()
{
	fflush(stderr);
}
#else
void info(char *fmt,...) {}
void info_flush() {}
#endif

// evaluate testing accuracy
double evaluate_testing(double *w, int n, const problem *prob_t)
{
	int correct=0;
	feature_node **tx = prob_t->x;
	int *ty = prob_t->y;
	for(int l =0; l < prob_t->l; l++)
	{
		double dec_value=0;
		int idx;
		for( const feature_node *lx=tx[l]; (idx=lx->index)!=-1; lx++)
		{
			if(idx<= n)
				dec_value += w[idx-1]*lx->value;
		}	

		if (dec_value > 0 && ty[l] == 1)
			++correct;
		else if (dec_value <=0 && ty[l] == -1)
			++correct;
	}
	return (double)correct/(double)prob_t->l;
}


class l2_lr_fun : public function
{
	public:
		l2_lr_fun(const problem *prob, const problem *prob_t, double Cp, double Cn);
		~l2_lr_fun();

		double fun(double *w);
		void grad(double *w, double *g);
		void Hv(double *s, double *Hs);
		double testing(double *w);

		int get_nr_variable(void);

	private:
		void Xv(double *v, double *Xv);
		void XTv(double *v, double *XTv);

		double *C;
		double *z;
		double *D;
		const problem *prob;
		const problem *prob_t;
};

l2_lr_fun::l2_lr_fun(const problem *prob, const problem *prob_t , double Cp, double Cn)
{
	int i;
	int l=prob->l;
	int *y=prob->y;

	this->prob = prob;
	this->prob_t = prob_t;

	z = new double[l];
	D = new double[l];
	C = new double[l];

	for (i=0; i<l; i++)
	{
		if (y[i] == 1)
			C[i] = Cp;
		else
			C[i] = Cn;
	}
}

l2_lr_fun::~l2_lr_fun()
{
	delete[] z;
	delete[] D;
	delete[] C;
}


double l2_lr_fun::fun(double *w)
{
	int i;
	double f=0;
	int *y=prob->y;
	int l=prob->l;
	int n=prob->n;

	Xv(w, z);
	for(i=0;i<l;i++)
	{
		double yz = y[i]*z[i];
		if (yz >= 0)
			f += C[i]*log(1 + exp(-yz));
		else
			f += C[i]*(-yz+log(1 + exp(yz)));
	}
	f = 2*f;
	for(i=0;i<n;i++)
		f += w[i]*w[i];
	f /= 2.0;

	return(f);
}

void l2_lr_fun::grad(double *w, double *g)
{
	int i;
	int *y=prob->y;
	int l=prob->l;
	int n=prob->n;

	for(i=0;i<l;i++)
	{
		z[i] = 1/(1 + exp(-y[i]*z[i]));
		D[i] = z[i]*(1-z[i]);
		z[i] = C[i]*(z[i]-1)*y[i];
	}
	XTv(z, g);

	for(i=0;i<n;i++)
		g[i] = w[i] + g[i];
}

int l2_lr_fun::get_nr_variable(void)
{
	return prob->n;
}

void l2_lr_fun::Hv(double *s, double *Hs)
{
	int i;
	int l=prob->l;
	int n=prob->n;
	double *wa = new double[l];

	Xv(s, wa);
	for(i=0;i<l;i++)
		wa[i] = C[i]*D[i]*wa[i];

	XTv(wa, Hs);
	for(i=0;i<n;i++)
		Hs[i] = s[i] + Hs[i];
	delete[] wa;
}

void l2_lr_fun::Xv(double *v, double *Xv)
{
	int i;
	int l=prob->l;
	feature_node **x=prob->x;

	for(i=0;i<l;i++)
	{
		feature_node *s=x[i];
		Xv[i]=0;
		while(s->index!=-1)
		{
			Xv[i]+=v[s->index-1]*s->value;
			s++;
		}
	}
}

void l2_lr_fun::XTv(double *v, double *XTv)
{
	int i;
	int l=prob->l;
	int n=prob->n;
	feature_node **x=prob->x;

	for(i=0;i<n;i++)
		XTv[i]=0;
	for(i=0;i<l;i++)
	{
		feature_node *s=x[i];
		while(s->index!=-1)
		{
			XTv[s->index-1]+=v[i]*s->value;
			s++;
		}
	}
}

double l2_lr_fun::testing(double *w)
{
	return evaluate_testing(w, prob->n, prob_t);
}

class l2loss_svm_fun : public function
{
	public:
		l2loss_svm_fun(const problem *prob, const problem *prob_t, double Cp, double Cn);
		~l2loss_svm_fun();

		double fun(double *w);
		void grad(double *w, double *g);
		void Hv(double *s, double *Hs);
		double testing(double *w);

		int get_nr_variable(void);

	private:
		void Xv(double *v, double *Xv);
		void subXv(double *v, double *Xv);
		void subXTv(double *v, double *XTv);

		double *C;
		double *z;
		double *D;
		int *I;
		int sizeI;
		const problem *prob;
		const problem *prob_t;
};

l2loss_svm_fun::l2loss_svm_fun(const problem *prob, const problem *prob_t, double Cp, double Cn)
{
	int i;
	int l=prob->l;
	int *y=prob->y;

	this->prob = prob;
	this->prob_t = prob_t;

	z = new double[l];
	D = new double[l];
	C = new double[l];
	I = new int[l];

	for (i=0; i<l; i++)
	{
		if (y[i] == 1)
			C[i] = Cp;
		else
			C[i] = Cn;
	}
}

l2loss_svm_fun::~l2loss_svm_fun()
{
	delete[] z;
	delete[] D;
	delete[] C;
	delete[] I;
}

double l2loss_svm_fun::fun(double *w)
{
	int i;
	double f=0;
	int *y=prob->y;
	int l=prob->l;
	int n=prob->n;

	Xv(w, z);
	for(i=0;i<l;i++)
	{
		z[i] = y[i]*z[i];
		double d = z[i]-1;
		if (d < 0)
			f += C[i]*d*d;
	}
	f = 2*f;
	for(i=0;i<n;i++)
		f += w[i]*w[i];
	f /= 2.0;

	return(f);
}

void l2loss_svm_fun::grad(double *w, double *g)
{
	int i;
	int *y=prob->y;
	int l=prob->l;
	int n=prob->n;

	sizeI = 0;
	for (i=0;i<l;i++)
		if (z[i] < 1)
		{
			z[sizeI] = C[i]*y[i]*(z[i]-1);
			I[sizeI] = i;
			sizeI++;
		}
	subXTv(z, g);

	for(i=0;i<n;i++)
		g[i] = w[i] + 2*g[i];
}

int l2loss_svm_fun::get_nr_variable(void)
{
	return prob->n;
}

void l2loss_svm_fun::Hv(double *s, double *Hs)
{
	int i;
	int l=prob->l;
	int n=prob->n;
	double *wa = new double[l];

	subXv(s, wa);
	for(i=0;i<sizeI;i++)
		wa[i] = C[I[i]]*wa[i];

	subXTv(wa, Hs);
	for(i=0;i<n;i++)
		Hs[i] = s[i] + 2*Hs[i];
	delete[] wa;
}

void l2loss_svm_fun::Xv(double *v, double *Xv)
{
	int i;
	int l=prob->l;
	feature_node **x=prob->x;

	for(i=0;i<l;i++)
	{
		feature_node *s=x[i];
		Xv[i]=0;
		while(s->index!=-1)
		{
			Xv[i]+=v[s->index-1]*s->value;
			s++;
		}
	}
}

void l2loss_svm_fun::subXv(double *v, double *Xv)
{
	int i;
	feature_node **x=prob->x;

	for(i=0;i<sizeI;i++)
	{
		feature_node *s=x[I[i]];
		Xv[i]=0;
		while(s->index!=-1)
		{
			Xv[i]+=v[s->index-1]*s->value;
			s++;
		}
	}
}

void l2loss_svm_fun::subXTv(double *v, double *XTv)
{
	int i;
	int n=prob->n;
	feature_node **x=prob->x;

	for(i=0;i<n;i++)
		XTv[i]=0;
	for(i=0;i<sizeI;i++)
	{
		feature_node *s=x[I[i]];
		while(s->index!=-1)
		{
			XTv[s->index-1]+=v[i]*s->value;
			s++;
		}
	}
}

double l2loss_svm_fun::testing(double *w)
{
	return evaluate_testing(w, prob->n, prob_t);
}

// A coordinate descent algorithm for 
// L2-loss SVM primal problems

// Given: 
// x, y, Cp, Cn
// eps is the stopping tolerance
//
// solution will be put in w

double recompute(const problem *, int *, double *, int *, double *, double, double, double *, int);

void l2_svm_cdprimal(const problem *prob, const double eps, double *w, double *x_val, int *x_ind, int *x_ptr,  double Cp, double Cn, function *fun_obj, int *index, int random_flag){

	// initial vaule
	int n = prob->n;
	int l = prob->l;
	double C = Cp;
	double *b = new double[l];
	int i, s, iter = 0;
	int col_i;
	double d, old_d, Dp, Dpmax, Dpp, loss, new_loss;
	double *tg = new double[n];
	const double sigma = 0.01;
	double *bound = new double[n];
	int *i_index = new int[n];
	int max_iter = 150;
	double *wt = new double[n];
	int total_time = 0;

	for(i=0; i<n; ++i)
		w[i] = 0;
	for(i=0; i<l; ++i)
		b[i] = 1;
	for(i=0; i<n; ++i)
		i_index[i] = i;


	for (i=0; i<n; i++ )
	{	
		double sum = 0;

		col_i=x_ptr[i];
		while(col_i < x_ptr[i+1])
		{
			sum += (x_val[col_i]*x_val[col_i]);
			col_i++;
		}
		bound[i] = (2*C*sum+1)/2+sigma;
	}

	while(iter< max_iter)
	{
		int clock_start_time = clock();
		Dpmax = 0;
		if(random_flag)
			for (i=0; i<n; i++)
			{
				int j = i+rand()%(n-i);
				swap(i_index[i], i_index[j]);
			}
		for(s=0; s<n; ++s){
			i = i_index[s];
			Dp = 0;
			Dpp = 0; 
			loss = 0;

			col_i = x_ptr[i];
			while( col_i <  x_ptr[i+1] )
			{
				double xji = x_val[col_i];
				int ind = x_ind[col_i];
				if (b[ind] > 0)
				{
					Dp -= b[ind]*xji*C ;
					Dpp += xji*xji*C;
					loss += b[ind]*b[ind]*C;
				}
				col_i++;
			}
			Dp = w[i] + 2*Dp;
			Dpp = 1 + 2*Dpp;

			if (fabs(Dp) > Dpmax)
				Dpmax = fabs(Dp);

			if (fabs(Dp/Dpp) <= 1e-12) 
				continue;

			d = -Dp/Dpp;
			old_d = 0;

			double ddiff;
			int step = 0;

			while(1){
				ddiff = old_d-d;
				step++;

				if ( Dp/d + bound[i] <= 0) 
				{
					col_i = x_ptr[i];
					while(col_i < x_ptr[i+1])
					{
						b[x_ind[col_i]] += ddiff*x_val[col_i];
						col_i++;
					}

					break;
				}

				// Recompute b if line search too many times 
				if (step % 10 == 0)
				{
					loss = recompute(prob, x_ind, x_val, x_ptr, w, Cp, Cn,b, i);
					col_i = x_ptr[i];
					while( col_i < x_ptr[i+1] )
					{
						b[x_ind[col_i]] -= old_d*x_val[col_i];
						col_i ++;
					}
				}
				new_loss = 0;

				for(col_i=x_ptr[i]; col_i < x_ptr[i+1]; ++col_i)
				{
					double tmp = b[x_ind[col_i]] + ddiff*x_val[col_i];
					b[x_ind[col_i]] = tmp;
					if (tmp> 0)
						new_loss += tmp*tmp*C;
				}
				old_d = d;
				if (w[i]*d + (0.5+sigma)*d*d + new_loss - loss <= 0)
					break;
				else
					d/=2;

			}

			w[i] += d;
		}
		iter++;
		total_time += clock() - clock_start_time;

		// evaluate function value, gradient (2-norm), and testing accuracy after each step
		for (int k=0; k<n; k++)
			wt[k] = w[index[k]];
		fun_obj->fun(wt);
		fun_obj->grad(wt,tg);
		int inc = 1;
		double gnorm2 = dnrm2_(&n, tg, &inc);
		printf("iter %d time %lf f %lf g %lf accuracy %lf \n",iter, (double)total_time/CLOCKS_PER_SEC,fun_obj->fun(wt),gnorm2, fun_obj->testing(wt));

		if(Dpmax < eps)
			break;
	}
	delete [] b;
	delete [] tg;
}


// recompute 

double recompute(const problem *prob, int *x_ind, double *x_val, int *x_ptr, double *w, double Cp, double Cn, double *b, int i)
{
	int l = prob->l;
	int n = prob->n;
	int col_i;
	for ( int j=0 ; j<l ; j++ )
		b[j] = 1;
	for ( int j=0 ; j<n ; j++ )
	{
		for(col_i=x_ptr[j]; col_i < x_ptr[j+1]; ++col_i)
		{
			b[x_ind[col_i]] -= w[j]*x_val[col_i];
		}
	}
	double loss = 0;
	for(col_i=x_ptr[i]; col_i < x_ptr[i+1]; ++col_i)
		if ( b[x_ind[col_i]] >0)
			loss += b[x_ind[col_i]]*b[x_ind[col_i]]*Cp;
	return loss;
}

//tansport Transport matrix X from row format to column format

void transport(const problem *prob, double **x_val, int **x_ind, int **x_ptr, int *index)
{
	int n= prob->n;
	int *y = prob->y;
	int *nl = new int[n];

	int i;
	for(i=0 ; i<n; i++)
		nl[i] = 0;
	int elements = 0;

	for(int i=0; i<prob->l; i++)
	{
		feature_node *s = prob->x[i];
		while(s->index != -1){
			int ind = index[s->index-1];	
			++nl[ind];
			++s;
			++elements;
		}
	}

	*x_val = Malloc(double, elements);
	*x_ind = Malloc(int, elements);
	*x_ptr = Malloc(int, n+1);

	int *t_ptr = Malloc(int, n+1);
	(*x_ptr)[0] = 0;	
	t_ptr[0] = 0;
	for(i=1; i< n+1 ;i++)
	{
		(*x_ptr)[i] = (*x_ptr)[i-1] + nl[i-1];
		t_ptr[i] = (*x_ptr)[i];
	}


	for(i=0; i<prob->l; i++)
	{
		feature_node *s = prob->x[i];	
		while(s->index !=-1)
		{
			int ind = index[s->index-1];	
			if(ind >= n || s->index-1 <0)
				fprintf(stderr,"%d %d %d\n", index[s->index-1],s->index-1,n);
			(*x_val)[t_ptr[ind]] = s->value*y[i];
			(*x_ind)[t_ptr[ind]] = i;
			++t_ptr[ind];
			++s;
		}
	}
}

// CMLS algorithm for 
// L2-loss SVM primal problems
// Only permute once before training
// Given: 
// x, y, Cp, Cn
// eps is the stopping tolerance
//
// solution will be put in w

void l2_svm_cmls(const problem *prob, const double eps, double *w, double *x_val, int *x_ind, int *x_ptr,  double Cp, double Cn, function *fun_obj, int random_flag)
{
	// initial vaule
	int n = prob->n;
	int l = prob->l;
	int iter = 1;
	double C = Cp;
	double *b = new double[l];
	int col_i;
	//double z, d, old_d, Dp, Dpp, loss, new_loss;
	double *tg = new double[n];
	double timer =0;
	double *delta = new double[n];
	double epsilon = 0.0001;
	int *index = new int[n];
	int max_iter = 150;
	for(int i=0; i<n; ++i)
		w[i] = 0;
	for ( int i=0 ; i<l ; i++ )
		b[i] = 1;
	for ( int i=0 ; i<n ; i++)
		delta[i] = 10.0;

	for(int i=0; i<n; i++)
		index[i] = i;

	for ( iter = 0 ; iter < max_iter ; iter++  ) 
	{
		double start = clock();
		double c = max(0.0 , 1.0 - (iter)/50.0);

		if(random_flag)
			for (int i=0; i<n; i++)
			{
				int j = i+rand()%(n-i);
				swap(index[i], index[j]);
			}

		for ( int s=0 ;s<n ; s++ )
		{
			int i = index[s];

			double sum_c = 0.0, sum_f = 0.0;
			for ( col_i=x_ptr[i]; col_i < x_ptr[i+1]; ++col_i)
			{
				int index = x_ind[col_i];
				double xji= x_val[col_i];
				if ( b[index] > 0)
					sum_c -= (b[index])*xji;
				else
					sum_c -= (b[index])*c*xji;
				if ( b[index] + fabs(delta[i]*xji) > 0)
					sum_f += xji*xji;
				else
					sum_f += c*xji*xji;
			}
			double delta_v_j = -(2*C*sum_c+w[i])/(sum_f*C*2+1);
			double delta_w_j = min(max(delta_v_j, -delta[i]), delta[i]);

			for ( col_i=x_ptr[i] ; col_i < x_ptr[i+1]; ++col_i)
			{
				b[x_ind[col_i]] -= delta_w_j*x_val[col_i];
			}
			w[i] += delta_w_j;
			delta[i] = 2*fabs(delta_w_j)+epsilon;
		}

		timer += clock() - start;

		//tesing after each step
		fun_obj->grad(w,tg);

		int inc = 1;
		double gnorm = dnrm2_(&n, tg, &inc);
		printf("iter %d time %lf f %lf g %lf accuracy %lf\n",iter, timer/CLOCKS_PER_SEC,fun_obj->fun(w),gnorm,  fun_obj->testing(w));

	}

	delete [] b;
	delete [] tg;

}

void cd_logistic(const problem *prob, double *w, double *x_val, int *x_ind, int *x_ptr,  double Cp, double Cn, function *fun_obj){
	// initial vaule
	int max_iter = 50;
	int n = prob->n;
	int l = prob->l;
	int *index = new int[n];
	int i, j, s, iter = 0;
	double C = Cp;
	double *b = new double[l];
	int col_i;
	double z, d, old_d, Dp, Dpp, loss, new_loss;
	double *tg = new double[n];
	double timer =0;
	const double sigma = 0.01;

	for(i=0; i<n; ++i)
		w[i] = 0;
	for(j=0; j<l; ++j)
		b[j] = 0;

	for(i=0; i<n; i++)
		index[i] = i;

	while(1)
	{
		double start = clock();
		if(iter >= max_iter)
			break;

		for (i=0; i<n; i++)
		{
			j = i+rand()%(n-i);
			swap(index[i], index[j]);
		}


		for(s=0; s<n; ++s)
		{
			i = index[s];
			z = w[i];
			Dp = 0;
			Dpp = 0; 
			loss = 0;

			for ( col_i = x_ptr[i]; col_i < x_ptr[i+1] ; col_i++)
			{
				double xji = x_val[col_i];
				int index = x_ind[col_i];
				double expb = exp(b[index]);
				double expbplus = 1/(expb+1);
				Dp -= xji*expbplus ;
				Dpp += xji*xji*expb*(expbplus*expbplus);
				loss += log(1+1/expb);
			}

			Dp = z + C*Dp;
			Dpp = 1 + C*Dpp;
			if (fabs(Dp/Dpp) <= 1.0e-7) 
				continue;
			d = -Dp/Dpp;

			old_d = 0;
			loss = loss*C;
			double ddiff;
			int step = 0;
			while(1)
			{
				ddiff = old_d-d;
				step++;

				// recompute b when some error occur
				if (step % 10 == 0)
				{
					for ( int k=0; k<l ; k++ )
						b[k] = 0;
					for ( int k=0; k<n ; k++ )
					{
						for ( col_i = x_ptr[k] ; col_i < x_ptr[k+1] ; col_i++)
							b[x_ind[col_i]] += w[k]*x_val[col_i];
					}
					loss = 0;

					for ( col_i = x_ptr[i]; col_i < x_ptr[i+1] ; col_i++)
					{
						double tmp = exp(b[x_ind[col_i]]);
						loss += log(1+1/tmp);
					}

					loss *= C;

					for ( col_i = x_ptr[i]; col_i < x_ptr[i+1] ; col_i++)
					{
						b[x_ind[col_i]] += old_d*x_val[col_i];
					}

				}
				new_loss = 0;

				for ( col_i = x_ptr[i]; col_i < x_ptr[i+1] ; col_i++ )
				{
					double tmp = b[x_ind[col_i]] - ddiff*x_val[col_i];
					b[x_ind[col_i]] -= ddiff*x_val[col_i];
					double tmpp= exp(tmp);
					new_loss += log(1+1/tmpp);
				}

				old_d = d;
				new_loss *= C;
				if (z*d + (0.5+sigma)*d*d + new_loss - loss <= 0)
					break;
				else
					d/=2;					
			}
			w[i] = z + d; 
		}
		iter++;

		timer += clock() - start;

		//tesing after each step
		fun_obj->grad(w,tg);

		int inc = 1;
		double gnorm = dnrm2_(&n, tg, &inc);
		printf("iter %d time %lf f %lf g %lf accuracy %lf\n",iter, timer/CLOCKS_PER_SEC,fun_obj->fun(w),gnorm,  fun_obj->testing(w));
	}

	delete [] b;
	delete [] tg;
}



/*
// label: label name, start: begin of each class, count: #data of classes, perm: indices to the original data
// perm, length l, must be allocated before calling this subroutine
void group_classes(const problem *prob, int *nr_class_ret, int **label_ret, int **start_ret, int **count_ret, int *perm)
{
	int l = prob->l;
	int max_nr_class = 16;
	int nr_class = 0;
	int *label = Malloc(int,max_nr_class);
	int *count = Malloc(int,max_nr_class);
	int *data_label = Malloc(int,l);
	int i;

	for(i=0;i<l;i++)
	{
		int this_label = prob->y[i];
		int j;
		for(j=0;j<nr_class;j++)
		{
			if(this_label == label[j])
			{
				++count[j];
				break;
			}
		}
		data_label[i] = j;
		if(j == nr_class)
		{
			if(nr_class == max_nr_class)
			{
				max_nr_class *= 2;
				label = (int *)realloc(label,max_nr_class*sizeof(int));
				count = (int *)realloc(count,max_nr_class*sizeof(int));
			}
			label[nr_class] = this_label;
			count[nr_class] = 1;
			++nr_class;
		}
	}

	int *start = Malloc(int,nr_class);
	start[0] = 0;
	for(i=1;i<nr_class;i++)
		start[i] = start[i-1]+count[i-1];
	for(i=0;i<l;i++)
	{
		perm[start[data_label[i]]] = i;
		++start[data_label[i]];
	}
	start[0] = 0;
	for(i=1;i<nr_class;i++)
		start[i] = start[i-1]+count[i-1];

	*nr_class_ret = nr_class;
	*label_ret = label;
	*start_ret = start;
	*count_ret = count;
	free(data_label);
}
*/
void train_one(const problem *prob, const problem *prob_t, const parameter *param, double *w, double Cp, double Cn)
{
	double eps=param->eps;
	int pos = 0;
	int neg = 0;
	for (int i=0; i<prob->l;i++)
		if (prob->y[i]==+1)
			pos++;
	neg = prob->l - pos;
	function *fun_obj=NULL;
	switch(param->solver_type)
	{
		case L2_LR:
			{
				fun_obj=new l2_lr_fun(prob, prob_t,Cp, Cn);
				TRON tron_obj(fun_obj, eps*min(pos,neg)/prob->l);
				tron_obj.tron(w);
				delete fun_obj;
				break;
			}
		case L2LOSS_SVM:
			{
				fun_obj=new l2loss_svm_fun(prob, prob_t, Cp, Cn);
				TRON tron_obj(fun_obj, eps*min(pos,neg)/prob->l);
				tron_obj.tron(w);
				delete fun_obj;
				break;
			}
		case L2LOSS_SVM_CDPRIMAL:
			{
				fun_obj=new l2loss_svm_fun(prob, prob_t, Cp, Cn);
				double *x_val;
				int *x_ind, *x_ptr;
				int *index = new int[prob->n];
				int i;

				for(i=0; i<prob->n; i++)
					index[i] = i;

				transport(prob, &x_val, &x_ind, &x_ptr, index);
				l2_svm_cdprimal(prob, eps, w, x_val, x_ind, x_ptr, Cp, Cn, fun_obj, index, 1);
				delete x_val;
				delete x_ptr;
				delete x_ind;
				delete fun_obj;
				break;
			}

		case L2LOSS_SVM_CDPRIMAL_PERMUTEONCE:
			{
				fun_obj=new l2loss_svm_fun(prob, prob_t, Cp, Cn);
				double *x_val;
				int *x_ind, *x_ptr;
				int *index = new int[prob->n];
				int i, j;

				for(i=0; i<prob->n; i++)
					index[i] = i;
		
				for( i=0;i<prob->n;i++){
					j = i+rand()%(prob->n-i);
					swap(index[i], index[j]);
				}

				transport(prob, &x_val, &x_ind, &x_ptr, index);
				l2_svm_cdprimal(prob, eps, w, x_val, x_ind, x_ptr, Cp, Cn, fun_obj, index, 0);
				delete x_val;
				delete x_ptr;
				delete x_ind;
				delete fun_obj;
				break;
			}

		case L2LOSS_SVM_CDPRIMAL_NOPERMUTE:
			{
				fun_obj=new l2loss_svm_fun(prob, prob_t, Cp, Cn);
				double *x_val;
				int *x_ind, *x_ptr;
				int *index = new int[prob->n];
				int i;

				for(i=0; i<prob->n; i++)
					index[i] = i;
		
				transport(prob, &x_val, &x_ind, &x_ptr, index);
				l2_svm_cdprimal(prob, eps, w, x_val, x_ind, x_ptr, Cp, Cn, fun_obj, index, 0);
				delete x_val;
				delete x_ptr;
				delete x_ind;
				delete fun_obj;
				break;
			}

		case L2LOSS_SVM_CMLS:
			{
				fun_obj=new l2loss_svm_fun(prob, prob_t, Cp, Cn);
				double *x_val;
				int *x_ind, *x_ptr;
				int *index = new int[prob->n];
				int i;

				for(i=0; i<prob->n; i++)
					index[i] = i;

				transport(prob, &x_val, &x_ind, &x_ptr, index);
				l2_svm_cmls(prob, eps, w, x_val, x_ind, x_ptr, Cp, Cn, fun_obj, 0);
				delete x_val;
				delete x_ptr;
				delete x_ind;
				delete fun_obj;
				break;
			}

		case L2LOSS_SVM_CMLS_PERMUTE:
			{
				fun_obj=new l2loss_svm_fun(prob, prob_t, Cp, Cn);
				double *x_val;
				int *x_ind, *x_ptr;
				int *index = new int[prob->n];
				int i;

				for(i=0; i<prob->n; i++)
					index[i] = i;

				transport(prob, &x_val, &x_ind, &x_ptr, index);
				l2_svm_cmls(prob, eps, w, x_val, x_ind, x_ptr, Cp, Cn, fun_obj, 1);
				delete x_val;
				delete x_ptr;
				delete x_ind;
				delete fun_obj;
				break;
			}

		case L2_LR_CD:
			{
				fun_obj=new l2_lr_fun(prob, prob_t,Cp, Cn);
				double *x_val;
				int *x_ind, *x_ptr;
				int *index = new int[prob->n];
				int i;

				for(i=0; i<prob->n; i++)
					index[i] = i;

				transport(prob, &x_val, &x_ind, &x_ptr, index);
				cd_logistic(prob, w, x_val, x_ind, x_ptr, Cp, Cn, fun_obj);
				delete fun_obj;
				delete x_val;
				delete x_ptr;
				delete x_ind;

				break;
			}


		default:
			{
				fprintf(stderr, "Error: unknown solver_type\n");
				break;
			}
	}
}

/*
//
// Interface functions
//
model* train(const problem *prob, const parameter *param)
{
	int i;
	int l = prob->l;
	int n = prob->n;
	model *model_ = Malloc(model,1);

	if(prob->bias>=0)
		model_->nr_feature=n-1;
	else
		model_->nr_feature=n;
	model_->param = *param;
	model_->bias = prob->bias;

	int nr_class;
	int *label = NULL;
	int *start = NULL;
	int *count = NULL;
	int *perm = Malloc(int,l);

	// group training data of the same class
	group_classes(prob,&nr_class,&label,&start,&count,perm);

	model_->nr_class=nr_class;
	model_->label = Malloc(int,nr_class);
	for(i=0;i<nr_class;i++)
		model_->label[i] = label[i];

	// calculate weighted C
	double *weighted_C = Malloc(double, nr_class);
	for(i=0;i<nr_class;i++)
		weighted_C[i] = param->C;
	for(i=0;i<param->nr_weight;i++)
	{
		int j;
		for(j=0;j<nr_class;j++)
			if(param->weight_label[i] == label[j])
				break;
		if(j == nr_class)
			fprintf(stderr,"warning: class label %d specified in weight is not found\n", param->weight_label[i]);
		else
			weighted_C[j] *= param->weight[i];
	}

	// constructing the subproblem
	feature_node **x = Malloc(feature_node *,l);
	for(i=0;i<l;i++)
		x[i] = prob->x[perm[i]];

	int k;
	problem sub_prob;
	sub_prob.l = l;
	sub_prob.n = n;
	sub_prob.x = Malloc(feature_node *,sub_prob.l);
	sub_prob.y = Malloc(int,sub_prob.l);

	for(k=0; k<sub_prob.l; k++)
		sub_prob.x[k] = x[k];

	if(nr_class==2)
	{
		model_->w=Malloc(double, n);

		int e0 = start[0]+count[0];
		k=0;
		for(; k<e0; k++)
			sub_prob.y[k] = +1;
		for(; k<sub_prob.l; k++)
			sub_prob.y[k] = -1;

		train_one(&sub_prob, NULL, param, &model_->w[0], weighted_C[0], weighted_C[1]);
	}
	else
	{
		model_->w=Malloc(double, n*nr_class);
		for(i=0;i<nr_class;i++)
		{
			int si = start[i];
			int ei = si+count[i];

			k=0;
			for(; k<si; k++)
				sub_prob.y[k] = -1;
			for(; k<ei; k++)
				sub_prob.y[k] = +1;
			for(; k<sub_prob.l; k++)
				sub_prob.y[k] = -1;

			train_one(&sub_prob,NULL, param, &model_->w[i*n], weighted_C[i], param->C);
		}
	}

	free(x);
	free(label);
	free(start);
	free(count);
	free(perm);
	free(sub_prob.x);
	free(sub_prob.y);
	free(weighted_C);
	return model_;
}

void destroy_model(struct model *model_)
{
	if(model_->w != NULL)
		free(model_->w);
	if(model_->label != NULL)
		free(model_->label);
	free(model_);
}

const char *solver_type_table[]=
{
	"L2_LR", "L2LOSS_SVM_DUAL", "L2LOSS_SVM","L1LOSS_SVM_DUAL", NULL
};

int save_model(const char *model_file_name, const struct model *model_)
{
	int i;
	int nr_feature=model_->nr_feature;
	int n;
	const parameter& param = model_->param;

	if(model_->bias>=0)
		n=nr_feature+1;
	else
		n=nr_feature;
	FILE *fp = fopen(model_file_name,"w");
	if(fp==NULL) return -1;

	int nr_classifier;
	if(model_->nr_class==2)
		nr_classifier=1;
	else
		nr_classifier=model_->nr_class;

	fprintf(fp, "solver_type %s\n", solver_type_table[param.solver_type]);
	fprintf(fp, "nr_class %d\n", model_->nr_class);
	fprintf(fp, "label");
	for(i=0; i<model_->nr_class; i++)
		fprintf(fp, " %d", model_->label[i]);
	fprintf(fp, "\n");

	fprintf(fp, "nr_feature %d\n", nr_feature);

	fprintf(fp, "bias %.16g\n", model_->bias);

	fprintf(fp, "w\n");
	for(i=0; i<n; i++)
	{
		int j;
		for(j=0; j<nr_classifier; j++)
			fprintf(fp, "%.16g ", model_->w[j*n+i]);
		fprintf(fp, "\n");
	}

	if (ferror(fp) != 0 || fclose(fp) != 0) return -1;
	else return 0;
}

struct model *load_model(const char *model_file_name)
{
	FILE *fp = fopen(model_file_name,"r");
	if(fp==NULL) return NULL;

	int i;
	int nr_feature;
	int n;
	int nr_class;
	double bias;
	model *model_ = Malloc(model,1);
	parameter& param = model_->param;

	model_->label = NULL;

	char cmd[81];
	while(1)
	{
		fscanf(fp,"%80s",cmd);
		if(strcmp(cmd,"solver_type")==0)
		{
			fscanf(fp,"%80s",cmd);
			int i;
			for(i=0;solver_type_table[i];i++)
			{
				if(strcmp(solver_type_table[i],cmd)==0)
				{
					param.solver_type=i;
					break;
				}
			}
			if(solver_type_table[i] == NULL)
			{
				fprintf(stderr,"unknown solver type.\n");
				free(model_->label);
				free(model_);
				return NULL;
			}
		}
		else if(strcmp(cmd,"nr_class")==0)
		{
			fscanf(fp,"%d",&nr_class);
			model_->nr_class=nr_class;
		}
		else if(strcmp(cmd,"nr_feature")==0)
		{
			fscanf(fp,"%d",&nr_feature);
			model_->nr_feature=nr_feature;
		}
		else if(strcmp(cmd,"bias")==0)
		{
			fscanf(fp,"%lf",&bias);
			model_->bias=bias;
		}
		else if(strcmp(cmd,"w")==0)
		{
			break;
		}
		else if(strcmp(cmd,"label")==0)
		{
			int nr_class = model_->nr_class;
			model_->label = Malloc(int,nr_class);
			for(int i=0;i<nr_class;i++)
				fscanf(fp,"%d",&model_->label[i]);
		}
		else
		{
			fprintf(stderr,"unknown text in model file: [%s]\n",cmd);
			free(model_);
			return NULL;
		}
	}

	nr_feature=model_->nr_feature;
	if(model_->bias>=0)
		n=nr_feature+1;
	else
		n=nr_feature;

	int nr_classifier;
	if(nr_class==2)
		nr_classifier = 1;
	else
		nr_classifier = nr_class;

	model_->w=Malloc(double, n*nr_classifier);
	for(i=0; i<n; i++)
	{
		int j;
		for(j=0; j<nr_classifier; j++)
			fscanf(fp, "%lf ", &model_->w[j*n+i]);
		fscanf(fp, "\n");
	}
	if (ferror(fp) != 0 || fclose(fp) != 0) return NULL;

	return model_;
}

int predict_values(const struct model *model_, const struct feature_node *x, double *dec_values)
{
	int idx;
	int n;
	if(model_->bias>=0)
		n=model_->nr_feature+1;
	else
		n=model_->nr_feature;
	double *w=model_->w;
	int nr_class=model_->nr_class;
	int i;
	int nr_classifier;
	if(nr_class==2)
		nr_classifier = 1;
	else
		nr_classifier = nr_class;
	for(i=0;i<nr_classifier;i++)
	{
		const feature_node *lx=x;
		double wtx=0;
		for(; (idx=lx->index)!=-1; lx++)
		{
			// the dimension of testing data may exceed that of training
			if(idx<=n)
				wtx += w[i*n+idx-1]*lx->value;
		}

		dec_values[i] = wtx;
	}

	if(nr_class==2)
		return (dec_values[0]>0)?model_->label[0]:model_->label[1];
	else
	{
		int dec_max_idx = 0;
		for(i=1;i<nr_class;i++)
		{
			if(dec_values[i] > dec_values[dec_max_idx])
				dec_max_idx = i;
		}
		return model_->label[dec_max_idx];
	}
}

int predict(const model *model_, const feature_node *x)
{
	double *dec_values = Malloc(double, model_->nr_class);
	int label=predict_values(model_, x, dec_values);
	free(dec_values);
	return label;
}

int predict_probability(const struct model *model_, const struct feature_node *x, double* prob_estimates)
{
	if(model_->param.solver_type==L2_LR)
	{
		int i;
		int nr_class=model_->nr_class;
		int nr_classifier;
		if(nr_class==2)
			nr_classifier = 1;
		else
			nr_classifier = nr_class;

		int label=predict_values(model_, x, prob_estimates);
		for(i=0;i<nr_classifier;i++)
			prob_estimates[i]=1/(1+exp(-prob_estimates[i]));

		if(nr_class==2) // for binary classification
			prob_estimates[1]=1.-prob_estimates[0];
		else
		{
			double sum=0;
			for(i=0; i<nr_class; i++)
				sum+=prob_estimates[i];

			for(i=0; i<nr_class; i++)
				prob_estimates[i]=prob_estimates[i]/sum;
		}

		return label;		
	}
	else
		return 0;
}
*/
void destroy_param(parameter* param)
{
	if(param->weight_label != NULL)
		free(param->weight_label);
	if(param->weight != NULL)
		free(param->weight);
}

const char *check_parameter(const problem *prob, const parameter *param)
{
	if(param->eps <= 0)
		return "eps <= 0";

	if(param->C <= 0)
		return "C <= 0";

	return NULL;
}
/*
void cross_validation(const problem *prob, const parameter *param, int nr_fold, int *target)
{
	int i;
	int *fold_start = Malloc(int,nr_fold+1);
	int l = prob->l;
	int *perm = Malloc(int,l);

	for(i=0;i<l;i++) perm[i]=i;
	for(i=0;i<l;i++)
	{
		int j = i+rand()%(l-i);
		swap(perm[i],perm[j]);
	}
	for(i=0;i<=nr_fold;i++)
		fold_start[i]=i*l/nr_fold;

	for(i=0;i<nr_fold;i++)
	{
		int begin = fold_start[i];
		int end = fold_start[i+1];
		int j,k;
		struct problem subprob;

		subprob.bias = prob->bias;
		subprob.n = prob->n;
		subprob.l = l-(end-begin);
		subprob.x = Malloc(struct feature_node*,subprob.l);
		subprob.y = Malloc(int,subprob.l);

		k=0;
		for(j=0;j<begin;j++)
		{
			subprob.x[k] = prob->x[perm[j]];
			subprob.y[k] = prob->y[perm[j]];
			++k;
		}
		for(j=end;j<l;j++)
		{
			subprob.x[k] = prob->x[perm[j]];
			subprob.y[k] = prob->y[perm[j]];
			++k;
		}
		struct model *submodel = train(&subprob,param);
		for(j=begin;j<end;j++)
			target[perm[j]] = predict(submodel,prob->x[perm[j]]);
		destroy_model(submodel);
		free(subprob.x);
		free(subprob.y);
	}
	free(fold_start);
	free(perm);
}

int get_nr_feature(const model *model_)
{
	return model_->nr_feature;
}

int get_nr_class(const model *model_)
{
	return model_->nr_class;
}

void get_labels(const model *model_, int* label)
{
	if (model_->label != NULL)
		for(int i=0;i<model_->nr_class;i++)
			label[i] = model_->label[i];
}
*/
